#import "BarcodeDemoViewController.h"
#import "AsBarcodeItem.h"
#import "BarcodeDemoTableViewCell.h"
#import "CSVData.h"
#import "SystemSetting.h"
#import "LongRange2DManager.h"

@interface BarcodeDemoViewController ()<LongRange2DManagerDelegate>
{
    BOOL isScanning;
}
@property (weak, nonatomic) IBOutlet UITableView *tableView;
@property (weak, nonatomic) IBOutlet UIBarButtonItem *outputBtn;
@property (weak, nonatomic) IBOutlet UIPickerView *pickerView;
@property (weak, nonatomic) IBOutlet UIView *rescanTimePickerTopView;
@property (weak, nonatomic) IBOutlet UITextField *rescanTimeTextField;
@property (weak, nonatomic) IBOutlet UIButton *startScanBtn;
@property (weak, nonatomic) IBOutlet UILabel *totalCountLabel;
@property (assign, nonatomic) long mTotalTagCounts;
@property (strong, nonatomic) IBOutletCollection(UIControl) NSArray *toDisableElems;
@property (strong, nonatomic) NSArray *rescanTimePickerData;
@property (strong, nonatomic) NSArray *rescanTimeData;
@property (readwrite, nonatomic) NSInteger rescanTimeValue;
@property (strong, nonatomic) NSMutableArray *barcodeData;
@property (strong, nonatomic) NSMutableDictionary *mapTags;
@property (strong, nonatomic) IBOutlet UISegmentedControl *encodingSegment;

- (IBAction)whenEncodingSegmentChanged:(id)sender;

@end

@implementation BarcodeDemoViewController{
    LongRange2DManager *longRange2DManager;
    NSStringEncoding encoding;
    BOOL isScanTimeOut;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    NSInteger decodingIndex = [[SystemSetting sharedSystemSetting] getDecodingIndex];
    [self.encodingSegment setSelectedSegmentIndex:decodingIndex];
    switch (decodingIndex) {
        case 0:
            encoding = NSASCIIStringEncoding;
            break;
        case 1:
            encoding = NSUTF8StringEncoding;
            break;
        case 2:
            encoding = NSShiftJISStringEncoding;
            break;
        default:
            encoding = NSASCIIStringEncoding;
            break;
    }
    longRange2DManager = [LongRange2DManager sharedLongRange2DManager];
    self.rescanTimeTextField.text = [self.rescanTimePickerData objectAtIndex:[[SystemSetting sharedSystemSetting] getBarcodeReScanTime]];
    self.rescanTimeValue = [[SystemSetting sharedSystemSetting] getBarcodeReScanTime];
    if ([[SystemSetting sharedSystemSetting] getBarcodeData]) {
        self.barcodeData = [[NSMutableArray alloc] initWithArray:[[SystemSetting sharedSystemSetting] getBarcodeData]];
    }else{
        self.barcodeData = [[NSMutableArray alloc]init];
    }
    [LongRange2DManager showGlobalProgressHUDWithTitle:@"Loading..."];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    if (self.barcodeData.count == 0) {
        [self.barcodeData removeAllObjects];
        _mapTags = [[NSMutableDictionary alloc] init];
    }else{
        _mapTags = [[NSMutableDictionary alloc] init];
        for (AsBarcodeItem * item in self.barcodeData) {
            [_mapTags setObject:item forKey:[NSString stringWithFormat:@"%@:%@", [LongRange2DBarcodeType getBarcodeString:item.barcodeType], item.barcode]];
        }
    }

    _mTotalTagCounts = 0;
    for (AsBarcodeItem * item in [_mapTags allValues]) {
        _mTotalTagCounts  =  _mTotalTagCounts +  item.count;
    }
    self.totalCountLabel.text = [NSString stringWithFormat:@"%lu", (unsigned long)_mTotalTagCounts];
}

- (void)viewDidAppear:(BOOL)animated
{
    [longRange2DManager  setDelegate:self];
    [LongRange2DManager dismissGlobalHUD];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    self->isScanning  = NO;
    [longRange2DManager setDelegate:nil];
    [longRange2DManager stopScan];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

#pragma mark - UITableViewDataSource
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.barcodeData.count;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    AsBarcodeItem *item = [self.barcodeData objectAtIndex:indexPath.row];
    CGRect rect = [item.barcode boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 65, 0) options:NSStringDrawingUsesLineFragmentOrigin attributes:@{NSFontAttributeName:[UIFont systemFontOfSize:14]} context:nil];
    return rect.size.height + 34;
}

#pragma mark - UITableViewDelegate
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    AsBarcodeItem *item = [self.barcodeData objectAtIndex:indexPath.row];
    NSString *cellIdentifier = @"BarcodeCell";
    BarcodeDemoTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifier];
    if(!cell) {
        cell = [[[NSBundle mainBundle] loadNibNamed:@"BarcodeDemoTableViewCell" owner:self options:nil] firstObject];
    }
    cell.codeTypeLabel.text = [LongRange2DBarcodeType getBarcodeString:item.barcodeType];
    cell.barcodeLabel.text = item.barcode;
    cell.countLabel.text = [NSString stringWithFormat:@"%d",item.count];
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    return cell;
}
- (UITableViewCellEditingStyle)tableView:(UITableView *)tableView editingStyleForRowAtIndexPath:(NSIndexPath *)indexPath {
    return UITableViewCellEditingStyleDelete;
}

#pragma mark - UIPickerViewDataSource
- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView
{
    return 1;
}
- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component
{
    return [self.rescanTimePickerData count];
}

#pragma mark - UIPickerViewDelegate
- (NSString *)pickerView:(UIPickerView *)pickerView titleForRow:(NSInteger)row forComponent:(NSInteger)component
{
    return [self.rescanTimePickerData objectAtIndex:row];
}

#pragma mark - IBActions
- (IBAction)clearBtnTapped:(id)sender {
    [self.barcodeData removeAllObjects];
    [self.mapTags removeAllObjects];
    [[SystemSetting sharedSystemSetting] setBarcodeData:self.barcodeData];
    [self.tableView reloadData];
    _mTotalTagCounts = 0;
    self.totalCountLabel.text = [NSString stringWithFormat:@"%lu", (unsigned long)_mTotalTagCounts];
}

- (IBAction)scanStartBtnTapped:(id)sender {
    if (isScanning != YES) {
        [self scanEnabled:YES];
    } else {
        [self scanEnabled:NO];
    }

}

- (IBAction)rescanTimeBtnTapped:(id)sender {
    self.pickerView.hidden = NO;
    [self.pickerView selectRow:self.rescanTimeValue inComponent:0 animated:NO];
    self.rescanTimePickerTopView.hidden = NO;
}

- (IBAction)pickerCloseBtnTapped:(id)sender {
    [self hidePickerView];
}

- (IBAction)rescanTimePickerOkBtnTapped:(id)sender {
    NSInteger idx = [self.pickerView selectedRowInComponent:0];
    self.rescanTimeTextField.text = [self.rescanTimePickerData objectAtIndex:idx];
    self.rescanTimeValue = idx;
    [[SystemSetting sharedSystemSetting] setBarcodeReScanTime:(int)self.rescanTimeValue];
    [self hidePickerView];
}

- (IBAction)outputCSVBtnTapped:(id)sender {
    dispatch_async(dispatch_get_main_queue(), ^{
        if (self.barcodeData && self.barcodeData.count > 0 ) {
            [CSVData isFileExist ];
            NSString *timestr = [self currentData];
            NSString *fileName = [NSString stringWithFormat:@"Barcode_%@.csv", timestr];
            [CSVData createCSVWithFileName:fileName];
            [CSVData exportBarCodeCSV:fileName withArray:self.barcodeData vc:self];
        } else {
            return;
        }
    });
}

- (NSString *)currentData {
    NSDate *today = [NSDate date];
    NSDateFormatter *df = [[NSDateFormatter alloc] init];
    [df setDateFormat:@"yyyyMMddHHmmss"];
    NSString *str = [df stringFromDate:today];
    return str;
}

#pragma mark - Getters
- (NSArray *)rescanTimePickerData {
    return @[@"Not Used", @"100 ms", @"200 ms", @"300 ms", @"400 ms", @"500 ms", @"600 ms", @"700 ms", @"800 ms", @"900 ms", @"1000 ms", @"1100 ms", @"1200 ms", @"1300 ms",
             @"1500 ms", @"2000 ms", @"2500 ms", @"3000 ms", @"3500 ms"];
}

- (NSArray *)rescanTimeData {
    return @[@"0", @"100", @"200", @"300", @"400", @"500",
             @"600", @"700", @"800", @"900", @"1000", @"1100", @"1200", @"1300", @"1500", @"2000", @"2500", @"3000",
             @"3500"];
}


#pragma mark - Custom
- (void)hidePickerView {
    self.pickerView.hidden = YES;
    self.rescanTimePickerTopView.hidden = YES;
}

- (void)scanEnabled:(BOOL)enabled {
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(restartScan) object:nil];
    if (enabled) {
        [longRange2DManager startScan];
        [self scanElemEnabled:NO];
    } else {
        [longRange2DManager stopScan];
        [self scanElemEnabled:YES];
    }
}

- (void)scanElemEnabled:(BOOL)enabled {
    if (enabled) {
        isScanning = NO;
        
        [self.startScanBtn setTitle:@"Start Scan" forState:UIControlStateNormal];
        
        for (UIControl *control in self.toDisableElems) {
            [control setEnabled:YES];
        }
        self.outputBtn.enabled = YES;
    } else {
        isScanning = YES;
        
        [self.startScanBtn setTitle:@"Stop Scan" forState:UIControlStateNormal];
        
        for (UIControl *control in self.toDisableElems) {
            [control setEnabled:NO];
        }
        self.outputBtn.enabled = NO;
    }
}

#pragma mark - Reader Event Handler Methods
- (void)updateDeviceState:(LongRange2DResultCode)error {
    if (error != LongRange2DResultNoError) {
        UIAlertController * alertController = [UIAlertController alertControllerWithTitle:@"Error" message:@"Failed to get response. Try again." preferredStyle:(UIAlertControllerStyleAlert)];
        UIAlertAction * action = [UIAlertAction actionWithTitle:@"OK" style:(UIAlertActionStyleDefault) handler:^(UIAlertAction * _Nonnull action) {
            [self.navigationController popViewControllerAnimated:YES];
        }];
        [alertController addAction:action];
        [self presentViewController:alertController animated:YES completion:nil];
    }
}

- (void)changedActionState:(LongRange2DCommandType)action resultCode:(NSInteger)resultCode {
    if(action == LongRange2DCommandStop)
    {
        int rescanTime = [[self.rescanTimeData objectAtIndex:self.rescanTimeValue] intValue];
        if (rescanTime > 0&&isScanning) {
            if (self->isScanTimeOut) {
                self->isScanTimeOut = NO;
                [self scanElemEnabled:YES];
                return ;
            }
            [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(restartScan) object:nil];
            [self performSelector:@selector(restartScan) withObject:nil afterDelay:rescanTime / 1000.00 ];
//            dispatch_after(dispatch_time(DISPATCH_TIME_NOW, rescanTime / 1000.00 * NSEC_PER_SEC), dispatch_get_main_queue(), ^(void){
//                if (self->isScanning){
//                    NSLog(@"开始扫描");
//                    [self->longRange2DManager.mLongRange2D startScan];
//                }
//            });
        } else {
            [self scanElemEnabled:YES];
        }
    }else if(action == LongRange2DCommandScanTimeOut){
        isScanTimeOut = YES;
        [self scanElemEnabled:YES];
    }
}
- (void)restartScan{
    if (self->isScanning){
        NSLog(@"开始扫描");
        [self->longRange2DManager startScan];
    }
}
- (void)receivedBarcodeData:(NSData *)barcodeData
                barcodeType:(LongRange2DBarcodeTypeOptions)barcodeType{
    NSString * barcode = [[NSString alloc] initWithData:barcodeData encoding:encoding];
    if (!barcode) {
        barcode = [[NSString alloc] initWithData:barcodeData encoding:NSASCIIStringEncoding];
    }
    if (barcodeType != LongRange2DBarcodeTypeNoRead) {
        _mTotalTagCounts++;
        NSDate *timeDate = [NSDate date];
        NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
        [formatter setDateStyle:NSDateFormatterMediumStyle];
        [formatter setTimeStyle:NSDateFormatterShortStyle];
        [formatter setDateFormat:@"YYYY/MM/dd HH:mm:ss"];
        NSString *dateTime = [formatter stringFromDate:timeDate];

        AsBarcodeItem *temItem = nil;
        if ((temItem = (AsBarcodeItem *)[self.mapTags objectForKey:[NSString stringWithFormat:@"%@:%@", [LongRange2DBarcodeType getBarcodeString:barcodeType], barcode]]) != nil) {
            temItem.count++;
            temItem.dateTime = dateTime;
        } else {
            AsBarcodeItem *item = [[AsBarcodeItem alloc] init];
            item.barcodeType = barcodeType;
            item.barcode = barcode;
            item.dateTime = dateTime;
            item.count = 1;
            [self.barcodeData insertObject:item atIndex:0];
            [_mapTags setObject:item forKey:[NSString stringWithFormat:@"%@:%@", [LongRange2DBarcodeType getBarcodeString:barcodeType], barcode]];
        }
        [[SystemSetting sharedSystemSetting] setBarcodeData:self.barcodeData];
        [self.tableView reloadData];
        self.totalCountLabel.text = [NSString stringWithFormat:@"%lu", (unsigned long)_mTotalTagCounts];
        self->isScanTimeOut = NO;
    }
}

- (void)receivedBarcodeString:(NSString *)barodeString
                  barcodeType:(LongRange2DBarcodeTypeOptions)barcodeType{
    
}
- (void)LongRange2DConnected:(BOOL)isConnected{
    if (!isConnected) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.navigationController popToRootViewControllerAnimated:YES];
        });
    }
}
- (IBAction)whenEncodingSegmentChanged:(id)sender {
    UISegmentedControl* control = (UISegmentedControl*)sender;
    switch (control.selectedSegmentIndex) {
        case 0:
            encoding = NSASCIIStringEncoding;
            break;
        case 1:
            encoding = NSUTF8StringEncoding;
            break;
        case 2:
            encoding = NSShiftJISStringEncoding;
            break;
        default:
            encoding = NSASCIIStringEncoding;
            break;
    }

    [[SystemSetting sharedSystemSetting] setDecodingIndex:control.selectedSegmentIndex];
}
- (BOOL)onAsReaderTriggerKeyEvent:(BOOL)status{
//    isScanning = status;
//    if(isScanning){
//        [self scanElemEnabled:NO];
//    }
//    return YES;
    [self scanEnabled:status];
    return NO;
}

@end
