//
//  BarcodeOption_VC.m
//  AsReaderLR
//
//  Created by mac on 2017/6/19.
//  Copyright © 2017年 asterisk. All rights reserved.
//

#import "BarcodeOption_VC.h"
#import "SystemSetting.h"
#import "LongRange2DManager.h"
#define PICKER_VIEW_BEEP          0
#define PICKER_VIEW_Vibrator        1

#define ALERT_VIEW_BATTERY_INTERVAL 10
#define ALERT_VIEW_SLEEP_TIME       11

@interface BarcodeOption_VC ()<LongRange2DManagerDelegate>

@property (weak, nonatomic) IBOutlet UIScrollView *scrollView;

@property (weak, nonatomic) IBOutlet UIPickerView *pickerView;
@property (weak, nonatomic) IBOutlet UITextField *buzzerTextField;
@property (weak, nonatomic) IBOutlet UITextField *vibratorTextField;
@property (weak, nonatomic) IBOutlet UITextField *batteryIntervalTextField;
@property (weak, nonatomic) IBOutlet UIView *buzzerPickerTopView;
@property (weak, nonatomic) IBOutlet UIView *vibratorPickerTopView;
@property (weak, nonatomic) IBOutlet UIView *shadowView;
@property (strong, nonatomic) NSArray *buzzerPickerData;
@property (strong, nonatomic) NSArray *vibratorPickerData;
@property (readwrite, nonatomic) LongRange2DBeepState beepValue;
@property (readwrite, nonatomic) LongRange2DVibratorState vibratorValue;
@property (readwrite, nonatomic) BOOL isCenterDecoding;
@property (readwrite, nonatomic) BOOL canUseCenterDecoding;
@property (strong, nonatomic) NSString *batteryIntervalValue;
@property (weak, nonatomic) IBOutlet UITextField *sleepTimeTextField;
@property (readwrite, nonatomic) NSInteger sleepTimeValue;
@property (strong, nonatomic) IBOutlet UITextField *autoOffTimeTextFiled;
@property (readwrite, nonatomic) NSInteger autoOffTimeValue;

@property (strong, nonatomic) IBOutlet UITextField *scannerTiemOutTextFiled;
@property (readwrite, nonatomic) NSInteger scannerTiemOutValue;
@property (weak, nonatomic) IBOutlet UILabel *centerDecodingLabel;
@property (weak, nonatomic) IBOutlet UISwitch *centerDecodingSwitch;
@property (weak, nonatomic) IBOutlet UIButton *saveButton;
@property (weak, nonatomic) IBOutlet UIButton *resetButton;

@end

@implementation BarcodeOption_VC{
    SystemSetting * systemSetting;
    LongRange2DManager *_longRange2DManager;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    self.scrollView.contentSize = CGSizeMake(0, 1000);
    self.scrollView.showsVerticalScrollIndicator = NO;
    systemSetting =[SystemSetting sharedSystemSetting];
    _longRange2DManager = [LongRange2DManager sharedLongRange2DManager];
    _canUseCenterDecoding = [[SystemSetting sharedSystemSetting] canUseCenterDecoding:[[LongRange2DManager sharedLongRange2DManager] getFirmwareVersion]];
    if (_canUseCenterDecoding) {
        self.centerDecodingLabel.hidden = NO;
        [self.centerDecodingSwitch setHidden:NO];
    }else {
        self.centerDecodingLabel.hidden = YES;
        [self.centerDecodingSwitch setHidden:YES];
        self.centerDecodingSwitch.enabled = NO;
        self.centerDecodingSwitch.userInteractionEnabled = NO;
        self.centerDecodingSwitch.on = NO;
    }
    
}


- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [_longRange2DManager setDelegate:self];
}

- (void)viewDidAppear:(BOOL)animated{
    [LongRange2DManager showGlobalProgressHUDWithTitle:@"Loading..."];
    appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    [_longRange2DManager setDelegate:self];
    if (_longRange2DManager.isConnected == NO) {
        [LongRange2DManager dismissGlobalHUD];
        return;
    }
    self.vibratorValue = [_longRange2DManager getVibrator];
    self.beepValue = [_longRange2DManager getBeep];
    self.sleepTimeValue = [_longRange2DManager getSleepTime];
    self.autoOffTimeValue = [_longRange2DManager getAutoOffTime];;
    self.scannerTiemOutValue = [_longRange2DManager getScannerTimeOut];
    if (_canUseCenterDecoding) {
        self.isCenterDecoding = _longRange2DManager.centerDecoding;
    }
    
    [self initViewData];
    [self.navigationController.view addSubview:self.shadowView];
    [LongRange2DManager dismissGlobalHUD];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    [_longRange2DManager  setDelegate:nil];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

#pragma mark - UIPickerViewDataSource
- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView
{
    return 1;
}
- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component
{
    if (pickerView.tag == PICKER_VIEW_BEEP) {
        return self.buzzerPickerData.count;
    }  else if(pickerView.tag == PICKER_VIEW_Vibrator){
        return self.vibratorPickerData.count;
    }
    return 0;
}

#pragma mark - UIPickerViewDelegate
- (NSString *)pickerView:(UIPickerView *)pickerView titleForRow:(NSInteger)row forComponent:(NSInteger)component
{
    if (pickerView.tag == PICKER_VIEW_BEEP) {
        return [self.buzzerPickerData objectAtIndex:row];
    } else if(pickerView.tag == PICKER_VIEW_Vibrator){
        return [self.vibratorPickerData objectAtIndex:row];
    }
    return @"None";
}

#pragma mark - Getters
- (NSArray *)buzzerPickerData {
    return @[@"Off", @"Low", @"High"];
}
- (NSArray *)vibratorPickerData {
    return @[@"Off", @"On"];
}

#pragma mark - IBAction
 - (IBAction)buzzerBtnTapped:(id)sender {
    self.pickerView.tag = PICKER_VIEW_BEEP;
    [self.pickerView reloadAllComponents];
    
    if (![self.buzzerTextField.text isEqualToString:@""]) {
        [self.pickerView selectRow:[self.buzzerPickerData indexOfObject:self.buzzerTextField.text] inComponent:0 animated:NO];
    } else {
        [self.pickerView selectRow:0 inComponent:0 animated:NO];
    }
    
    self.buzzerPickerTopView.hidden = NO;
    [self showPickerView];
}
- (IBAction)vibratorBtnTapped:(id)sender {
    
    self.pickerView.tag = PICKER_VIEW_Vibrator;
    [self.pickerView reloadAllComponents];
    
    if (![self.vibratorTextField.text isEqualToString:@""]) {
        [self.pickerView selectRow:[self.vibratorPickerData indexOfObject:self.vibratorTextField.text] inComponent:0 animated:NO];
    } else {
        [self.pickerView selectRow:0 inComponent:0 animated:NO];
    }
    
    self.vibratorPickerTopView.hidden = NO;
    [self showPickerView];
}

- (IBAction)batteryIntervalBtnTapped:(id)sender {
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:@"Battery Interval:" message:@"Please enter battery interval(0~30)." preferredStyle:UIAlertControllerStyleAlert];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField *textField){
      textField.keyboardType = UIKeyboardTypeNumberPad;
    }];
    
    UIAlertAction *okAction = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:^(UIAlertAction *action) {
        UITextField *battery = alertController.textFields.firstObject;
        if (battery.text.length == 0){
            battery.text = @"0";
        }
        int time = [battery.text intValue];
        if (time > 30 || time < 0) {
            if (time < 0) {
                battery.text = @"0";
            }
            if (time > 30) {
                battery.text = @"30";
            }
            UIAlertController * alertController = [UIAlertController alertControllerWithTitle:@"Battery Interval Value" message:@"Please enter the value (0~30)"  preferredStyle:UIAlertControllerStyleAlert];
            UIAlertAction * actionCancel = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleCancel handler:nil];
            [alertController addAction:actionCancel];
            [self.navigationController presentViewController:alertController animated:NO completion:nil];
        }

        [self setBatteryInterval:battery.text];

    }];
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:^(UIAlertAction *action) {
      
    }];
    [alertController addAction:okAction];
    [alertController addAction:cancelAction];
    [self presentViewController:alertController animated:YES completion:nil];
}
- (IBAction)sleepTimeBtnTapped:(id)sender{
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:@"Sleep Time:" message:@"Please enter sleep time(0~1800)." preferredStyle:UIAlertControllerStyleAlert];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField *textField){
        textField.keyboardType = UIKeyboardTypeNumberPad;
    }];
    
    UIAlertAction *okAction = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:^(UIAlertAction *action) {
        UITextField *sleepTime = alertController.textFields.firstObject;
        int time = [sleepTime.text intValue];
        if (time == 0){
            sleepTime.text = @"0";
        } else if (time > 1800 || time < 30) {
            if (time < 30) {
                sleepTime.text = @"30";
            }
            if (time > 1800) {
                sleepTime.text = @"1800";
            }
        }
        [self setSleepTime:sleepTime.text];
        
    }];
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:^(UIAlertAction *action) {
        
    }];
    [alertController addAction:okAction];
    [alertController addAction:cancelAction];
    [self presentViewController:alertController animated:YES completion:nil];
}
- (IBAction)scannerTimeOutBtnTapped:(id)sender {
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:@"Scanner Time Out:" message:@"Please enter scanner time out (4~30)." preferredStyle:UIAlertControllerStyleAlert];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField *textField){
        textField.keyboardType = UIKeyboardTypeNumberPad;
    }];

    UIAlertAction *okAction = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:^(UIAlertAction *action) {
        UITextField *scannerTimeOut = alertController.textFields.firstObject;
        if (scannerTimeOut.text.length == 0){
            scannerTimeOut.text = @"0";
        }
        int time = [scannerTimeOut.text intValue];
        if (time > 30 || time < 4) {
            if (time < 4) {
                scannerTimeOut.text = @"4";
            }
            if (time > 30) {
                scannerTimeOut.text = @"30";
            }
            UIAlertController * alertController = [UIAlertController alertControllerWithTitle:@"Scanner Time Out Value" message:@"Please enter the value (4~30)"  preferredStyle:UIAlertControllerStyleAlert];
            UIAlertAction * actionCancel = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleCancel handler:nil];
            [alertController addAction:actionCancel];
            [self.navigationController presentViewController:alertController animated:NO completion:nil];
        }
        [self setScannerTimeOut:scannerTimeOut.text];

    }];
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:^(UIAlertAction *action) {

    }];
    [alertController addAction:okAction];
    [alertController addAction:cancelAction];
    [self presentViewController:alertController animated:YES completion:nil];
}
- (IBAction)autoOffTimeBtnTapped:(id)sender {
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:@"Auto Off Time:" message:@"Please enter auto off time (0~300)." preferredStyle:UIAlertControllerStyleAlert];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField *textField){
        textField.keyboardType = UIKeyboardTypeNumberPad;
    }];
    
    UIAlertAction *okAction = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:^(UIAlertAction *action) {
        UITextField *autoOffTime = alertController.textFields.firstObject;
        if (autoOffTime.text.length == 0){
            autoOffTime.text = @"0";
        }
        int time = [autoOffTime.text intValue];
        if (time > 300 || time < 0) {
            if (time < 0) {
                autoOffTime.text = @"0";
            }
            if (time > 300) {
                autoOffTime.text = @"300";
            }
            UIAlertController * alertController = [UIAlertController alertControllerWithTitle:@"Auto Off Time Value" message:@"Please enter the value (0~300)"  preferredStyle:UIAlertControllerStyleAlert];
            UIAlertAction * actionCancel = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleCancel handler:nil];
            [alertController addAction:actionCancel];
            [self.navigationController presentViewController:alertController animated:NO completion:nil];
        }
        [self setAutoOffTime:autoOffTime.text];
        
    }];
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:^(UIAlertAction *action) {
        
    }];
    [alertController addAction:okAction];
    [alertController addAction:cancelAction];
    [self presentViewController:alertController animated:YES completion:nil];
}

- (IBAction)saveBtnTapped:(id)sender {
    [_longRange2DManager setBeep:self.beepValue];
    [_longRange2DManager setVibrator:self.vibratorValue ];
    [_longRange2DManager setSleepTime:(int)self.sleepTimeValue ];
    [_longRange2DManager setAutoOffTime:(int) self.autoOffTimeValue];
    [_longRange2DManager setScannerTimeOut:(int) self.scannerTiemOutValue];
    if (_canUseCenterDecoding) {
        [_longRange2DManager setCenterDecoding:self.isCenterDecoding];
        [systemSetting setCenterDecoding:self.isCenterDecoding];
    }
    [systemSetting setBeep:(int)self.beepValue];
    [systemSetting setVibrator:(int)self.vibratorValue];
    [systemSetting setSleepTime:(int)self.sleepTimeValue];
    [systemSetting setAutoOffTime:(int)self.autoOffTimeValue];
    [systemSetting setBatteryInterval:self.batteryIntervalValue];
    [systemSetting setScannerTiemOut:(int)self.scannerTiemOutValue];
   
    UIAlertController * alertController = [UIAlertController alertControllerWithTitle:@"Success" message:@"Saved successfully" preferredStyle:(UIAlertControllerStyleAlert)];
    UIAlertAction * action = [UIAlertAction actionWithTitle:@"OK" style:(UIAlertActionStyleDefault) handler:^(UIAlertAction * _Nonnull action) {
        
    }];
    [alertController addAction:action];
    [self presentViewController:alertController animated:YES completion:nil];
}

- (IBAction)onClickResetFactoryDefaults:(id)sender {
    [_longRange2DManager setDefaultScannerPrama];
    [_longRange2DManager setBeep:LongRange2DBeep_High];
    [_longRange2DManager setVibrator:LongRange2DVibrator_On];
    [_longRange2DManager setSleepTime:300];
    [_longRange2DManager setAutoOffTime:300];
    [_longRange2DManager setScannerTimeOut:30];
    [_longRange2DManager setCenterDecoding:NO];
    self.vibratorValue = [_longRange2DManager getVibrator];
    self.beepValue = [_longRange2DManager getBeep];
    self.sleepTimeValue = [_longRange2DManager getSleepTime];
    self.autoOffTimeValue = [_longRange2DManager getAutoOffTime];;
    self.scannerTiemOutValue = [_longRange2DManager getScannerTimeOut];
    if (_canUseCenterDecoding) {
        self.isCenterDecoding = _longRange2DManager.centerDecoding;
        [systemSetting setCenterDecoding:self.isCenterDecoding];
    }
    [systemSetting setBeep:(int)self.beepValue];
    [systemSetting setVibrator:(int)self.vibratorValue];
    [systemSetting setSleepTime:(int)self.sleepTimeValue];
    [systemSetting setAutoOffTime:(int)self.autoOffTimeValue];
    [systemSetting setScannerTiemOut:(int)self.scannerTiemOutValue];
    [systemSetting setBatteryInterval:DEFAULT_BATTERY_INTERVAL];
   
    [self initViewData];
    UIAlertController * alertController = [UIAlertController alertControllerWithTitle:@"Success" message:@"Set to default successfully" preferredStyle:(UIAlertControllerStyleAlert)];
    UIAlertAction * action = [UIAlertAction actionWithTitle:@"OK" style:(UIAlertActionStyleDefault) handler:^(UIAlertAction * _Nonnull action) {

    }];
    [alertController addAction:action];
    [self presentViewController:alertController animated:YES completion:nil];
}

- (IBAction)pickerCloseBtnTapped:(id)sender {
    [self hidePickerView];
}

- (IBAction)buzzerPickerOkBtnTapped:(id)sender {
    NSInteger idx = [self.pickerView selectedRowInComponent:0];
    [self setBeep:idx];
    [self hidePickerView];
}
- (IBAction)vibratorPickerOkBtnTapped:(id)sender {
    NSInteger idx = [self.pickerView selectedRowInComponent:0];
    [self setVibrator:idx];
    [self hidePickerView];
}
- (IBAction)tableCloseBtnTapped:(id)sender {
    [self hideTableView];
}


- (IBAction)onClickDecodingSwitch:(UISwitch *)sender {
    self.isCenterDecoding = sender.isOn;
}


#pragma mark - Custom
- (void)showPickerView {
    self.pickerView.hidden = NO;
    self.shadowView.hidden = NO;
}
- (void)hidePickerView {
    self.vibratorPickerTopView.hidden = YES;
    self.buzzerPickerTopView.hidden = YES;
    self.pickerView.hidden = YES;
    self.shadowView.hidden = YES;
}
- (void)hideTableView {
    self.shadowView.hidden = YES;
}
- (void)setBeep:(NSInteger)index {
    self.buzzerTextField.text = [self.buzzerPickerData objectAtIndex:index];
    self.beepValue = (int)index;
}
- (void)setVibrator:(NSInteger)index {
    self.vibratorTextField.text = [self.vibratorPickerData objectAtIndex:index];
    self.vibratorValue = (int)index;
}
- (void)setBatteryInterval:(NSString *)batteryInterval {
    self.batteryIntervalTextField.text = batteryInterval;
    self.batteryIntervalValue = batteryInterval;
}
- (void)setSleepTime:(NSString *)sleepTime {
    self.sleepTimeTextField.text = sleepTime;
    self.sleepTimeValue = [sleepTime integerValue];
}
- (void)setAutoOffTime:(NSString *)autoOffTime {
    self.autoOffTimeTextFiled.text = autoOffTime;
    self.autoOffTimeValue = [autoOffTime integerValue];
}
- (void)setScannerTimeOut:(NSString *)scannerTimeOut {
    self.scannerTiemOutTextFiled.text = scannerTimeOut;
    self.scannerTiemOutValue = [scannerTimeOut integerValue];
}
- (void)initViewData {
    [self setBeep:self.beepValue];
    [self setVibrator:self.vibratorValue];
    [self setSleepTime:[NSString stringWithFormat:@"%ld", (long)self.sleepTimeValue]];
    [self setAutoOffTime:[NSString stringWithFormat:@"%ld", (long)self.autoOffTimeValue]];
    [self setScannerTimeOut:[NSString stringWithFormat:@"%ld", (long)self.scannerTiemOutValue]];
    if (self.canUseCenterDecoding) {
        self.centerDecodingSwitch.on = self.isCenterDecoding;
    }
    NSString *batteryInterval = [systemSetting getBatteryInterval];
    [self setBatteryInterval:batteryInterval];
}

#pragma mark - Reader Event Handler Methods
- (void)LongRange2DConnected:(BOOL)isConnected{
    if (!isConnected) {
        [self hideTableView];
        [self hidePickerView];
        [self.navigationController popToRootViewControllerAnimated:YES];
    }
}

- (void)updateDeviceState:(LongRange2DResultCode)error {
    if (error != LongRange2DResultNoError) {
        UIAlertController * alertController = [UIAlertController alertControllerWithTitle:@"Error" message:@"Failed to get response. Try again." preferredStyle:(UIAlertControllerStyleAlert)];
        UIAlertAction * action = [UIAlertAction actionWithTitle:@"OK" style:(UIAlertActionStyleDefault) handler:^(UIAlertAction * _Nonnull action) {
            [self.navigationController popViewControllerAnimated:YES];
        }];
        [alertController addAction:action];
        [self presentViewController:alertController animated:YES completion:nil];
    }
}

@end
