//
//  FirmwareUpdateManager.m
//  LongRange2DDemo
//
//  Created by mac on 2020/10/28.
//  Copyright © 2020 zyl. All rights reserved.
//

#import "FirmwareUpdateManager.h"

#define PACKET_PREFIX                   0x7e          // '~'
#define PACKET_SUFFIX1                  0x0d          // '\r'
#define PACKET_SUFFIX2                  0x0a          // '\n'

#define PACKET                          256.0
@interface FirmwareUpdateManager ()
{
    NSMutableData *_mRecvData;
    BOOL updateFirmwareStatus;
    LongRange2DPacketManager * mPacket;
}
@end
@implementation FirmwareUpdateManager

static FirmwareUpdateManager * sharedFirmwareUpdateManager;
+ (FirmwareUpdateManager *)sharedFirmwareUpdateManager{
    if (!sharedFirmwareUpdateManager) {
        sharedFirmwareUpdateManager = [[FirmwareUpdateManager alloc] init];
    }
    return sharedFirmwareUpdateManager;
}
- (instancetype)init
{
    self = [super init];
    if (self) {
        updateFirmwareStatus = NO;
        mPacket = [[LongRange2DPacketManager alloc] init];
    }
    return self;
}
- (void)updateFirmwareWithFileName:(NSString *)fileName
                          fileData:(NSData *)data
                            result:(void(^)(BOOL result))result
                          progress:(void(^)(NSProgress * progress))progress{
    @try {
        if (!data || data.length == 0 || data.length % 2 != 0) {
            result(NO);
            return;
        }
        
        if (![LongRange2DManager sharedLongRange2DManager].isConnected) {
            result(NO);
            return;
        }
        
        [self startUpdateFirmware];
        //获取地址
        NSString *address = [self getFirmwareAddress];
        if (!address || [address isEqualToString:@""]) {
            result(NO);
            [self stopUpdateFirmware];
            return;
        }
        NSData *sendFileData = [[NSData alloc] init];
        if([@"0000+8001000" isEqualToString:address]){
            //文件后半部分数据传输
            sendFileData = [data subdataWithRange:NSMakeRange(data.length/2, data.length/2)];
        }else if([@"0000+8010000" isEqualToString:address]){
            //文件前半部分数据传输
            sendFileData = [data subdataWithRange:NSMakeRange(0, data.length/2)];
        }else{
            result(NO);
            [self stopUpdateFirmware];
            return;
        }
        if (sendFileData.length == 0) {
            result(NO);
            [self stopUpdateFirmware];
            return;
        }
        NSString *fileSize = 0;
        fileSize = [NSString stringWithFormat:@"%lu",(unsigned long)sendFileData.length];
        int countPicket =  ceilf(sendFileData.length/PACKET);
        NSLog(@"countPicket : %d",countPicket);
        NSProgress * prog = [NSProgress progressWithTotalUnitCount:countPicket];

        //发送固件信息
        NSData *sendData = [[NSData alloc] initWithData:[[NSString stringWithFormat:@"%@%@", fileName, fileSize] dataUsingEncoding:NSASCIIStringEncoding]];
        BOOL updateFirmwareInfo = [self writeFirmwareData:sendData Property:LongRange2DFirmwareTypeInfo];
        if (updateFirmwareInfo == NO) {
            result(NO);
            [self stopUpdateFirmware];
            return;
        }
        int resend = 0;
        int picketNumber = -1;
        int resendNumber = 0;
        for (int i = 0 ; i < countPicket ; i++) {
            NSData * sendData = nil;
            if (sendFileData.length > i*PACKET + PACKET) {
                sendData = [sendFileData subdataWithRange:NSMakeRange( i*PACKET, PACKET)];
            }else{
                sendData = [sendFileData subdataWithRange:NSMakeRange( i*PACKET, sendFileData.length - i*PACKET)];
            }
            picketNumber ++;
            if (picketNumber > 255) {
                picketNumber = 0;
            }
            NSLog(@"i : %d",i);

            NSLog(@"picketNumber : %d",picketNumber);
            if (![LongRange2DManager sharedLongRange2DManager].isConnected) {
                NSLog(@"_longRange2D is not connected");
                result(NO);
                [self stopUpdateFirmware];
                return;
            }
            NSMutableData *sendPicketData = [[NSMutableData alloc] init];
            Byte picket[1] = {static_cast<Byte>(picketNumber)};
            [sendPicketData appendData:[NSData dataWithBytes:picket length:1]];
            [sendPicketData appendData:sendData];
            BOOL updateFirmwarePickety = [self writeFirmwareData:sendPicketData Property:LongRange2DFirmwareTypePicket];
            [NSThread sleepForTimeInterval:0.05];
            if (updateFirmwarePickety == NO) {
                if (resendNumber != i) {
                    resendNumber = i;
                    resend = 0;
                }
                i--;
                picketNumber--;
                resend++;
                if (resend > 4) {
                    result(NO);
                    [self stopUpdateFirmware];
                    return;
                }
            } else {
                prog.completedUnitCount++;
                progress(prog);
            }
        }
        //固件更新结束
        BOOL updateFirmwarePicketEND = [self writeFirmwareData:nil Property:LongRange2DFirmwareTypePicketEND];
        [NSThread sleepForTimeInterval:0.05];
        if (updateFirmwarePicketEND == NO) {
            result(NO);
            [self stopUpdateFirmware];
            return;
        } else {
            [prog setCompletedUnitCount:countPicket];
            progress(prog);
            result(YES);
            [self stopUpdateFirmware];
        }
    } @catch (NSException *exception) {
        result(NO);
        [self stopUpdateFirmware];
    } @finally {
    }
}
- (void)rX:(NSData *)readData {
    if (updateFirmwareStatus) {
        if (readData && readData.length > 0) {
            [self readData:readData];
        }
    }
}
- (BOOL)tX:(NSData *)transmitData {
    if (!updateFirmwareStatus) {
        return NO;
    }
    return [[LongRange2DManager sharedLongRange2DManager] sendData:transmitData];
}
- (void)startUpdateFirmware {
    updateFirmwareStatus = YES;
}
- (void)stopUpdateFirmware {
    updateFirmwareStatus = NO;
}
- (NSString *)getFirmwareAddress {
    _mRecvData = [[NSMutableData alloc] init];
    NSString *firmwareAddress = @"~ra\r\n";
    NSData *sendData = [firmwareAddress dataUsingEncoding:NSASCIIStringEncoding];
    [self tX:sendData];
    LongRange2DPacket *packet = [[LongRange2DPacket alloc]initWithPacketType:LongRange2DPacketResGetProperty subType:LongRange2DFirmwareTypeAddress];
    [self wite:packet];
    LongRange2DResultCode resCode = (LongRange2DResultCode)packet.errorCode;
    if (resCode != LongRange2DResultNoError) {
        return @"";
    }
    NSData *resData = packet.data;
    NSString *firmwareAddressString = [[NSString alloc] initWithData:resData encoding:NSASCIIStringEncoding];
    return firmwareAddressString;
}
- (BOOL)writeFirmwareData:(NSData *)data Property:(LongRange2DFirmwareType)property {
    _mRecvData = [[NSMutableData alloc] init];
    NSString *propertyString = [NSString stringWithFormat:@"~w%c", property];
    NSString *endData = @"\r\n";
    NSMutableData *sendData = [[NSMutableData alloc] initWithData:[propertyString dataUsingEncoding:NSASCIIStringEncoding]];
    if (data) {
        [sendData appendData:data];
    }
    [sendData appendData:[endData dataUsingEncoding:NSASCIIStringEncoding]];
    BOOL tXResult = [self tX:sendData];
    if (!tXResult) {
        return NO;
    }
    LongRange2DPacket *packet = [[LongRange2DPacket alloc]initWithPacketType:LongRange2DPacketResSetProperty subType:property];

    [self wite:packet];
    LongRange2DResultCode resCode = (LongRange2DResultCode)packet.errorCode;
    if (resCode != LongRange2DResultNoError) {
        return NO;
    } else {
        return YES;
    }
}
- (void)wite:(LongRange2DPacket *)packet{
    [mPacket addPacket:packet];
    [NSThread detachNewThreadSelector:@selector(changePacket:) toTarget:self withObject:packet];
    while (!packet.data) {
        if (![LongRange2DManager sharedLongRange2DManager].isConnected) {
            break;
        }
        [NSThread sleepForTimeInterval:0.01];
        [[NSRunLoop currentRunLoop] runMode:NSDefaultRunLoopMode beforeDate:[NSDate distantFuture]];
    }
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(changePacket:) object:packet];
    [mPacket removePacket:packet];

}

- (void)changePacket:(LongRange2DPacket *)packet{
    [NSThread sleepForTimeInterval:2.0];
    [self performSelectorOnMainThread:@selector(cPacket:) withObject:packet waitUntilDone:NO];
}

- (void)cPacket:(LongRange2DPacket *)packet{
    if (!packet.data) {
        packet.data = [[NSData alloc] init];
        packet.errorCode = LongRange2DResultOtherError;
    }
}
- (void)readData:(NSData *)data {
    [_mRecvData appendData:data];
    NSData *recvData = nil;
    NSRange range;
    while(true)
    {
        range = [self findDataWidthRange:_mRecvData withPrefix:PACKET_PREFIX withSuffix1:PACKET_SUFFIX1 withSuffix2:PACKET_SUFFIX2];
        if (range.location == NSNotFound && range.length == NSNotFound) {
            return;
        }
        if (range.location != NSNotFound && range.length != NSNotFound) {
            if (range.length < 6) {
                [_mRecvData replaceBytesInRange:range withBytes:NULL length:0];
                return;
            } else {
                recvData = [_mRecvData subdataWithRange:NSMakeRange(range.location + 1, range.length - 3)];
                [_mRecvData replaceBytesInRange:range withBytes:NULL length:0];
            }
        } else if (range.location != NSNotFound && range.length == NSNotFound) {
            if (range.location > 0) {
                range.length = range.location - 1;
                range.location = 0;
                [_mRecvData replaceBytesInRange:range withBytes:NULL length:0];
            }
            return;
        } else if (range.location == NSNotFound && range.length != NSNotFound) {
            range.location = 0;
            [_mRecvData replaceBytesInRange:range withBytes:NULL length:0];
            return;
        } else {
            return;
        }

        if (recvData.length < 2) {
            return;
        }
        uint8_t *totalPack = (uint8_t *)[recvData bytes];
        uint8_t pack = totalPack[0];
        uint8_t subPack = totalPack[1];
        NSInteger errorCode = [self getErrorCode:recvData findPosition:2];
        NSData *value = nil;
        int offset = 2;
        if (recvData.length > 2) {
            value = [recvData subdataWithRange:NSMakeRange(offset, recvData.length - offset)];
        }
        switch (pack) {
            case LongRange2DFirmwarePropertyTypeGet://R
                switch (subPack) {
                    case LongRange2DFirmwareTypeAddress://a
                    {
                        LongRange2DPacket *packet = [mPacket findPacket:LongRange2DPacketResGetProperty subType:subPack];
                        if (packet != nil) {
                            packet.errorCode = errorCode;
                            packet.data = value;
                            [packet signal];
                        }
                    }
                        break;
                    default:
                        break;
                }
                break;
            case LongRange2DFirmwarePropertyTypeSet://W
                switch (subPack) {
                    case LongRange2DFirmwareTypeInfo://i
                    case LongRange2DFirmwareTypePicket://f
                    case LongRange2DFirmwareTypePicketEND: //d
                    {
                        LongRange2DPacket *packet = [mPacket findPacket:LongRange2DPacketResSetProperty subType:subPack];
                        if (packet != nil) {
                            packet.errorCode = errorCode;
                            packet.data = value;
                            [packet signal];
                        }
                    }
                        break;
                    default:
                       
                        break;
                }
                break;
            default:
                break;
               
        }
    }
}
- (NSRange)findDataWidthRange:(NSData *)data withPrefix:(uint8_t)prefix withSuffix1:(uint8_t)suffix1 withSuffix2:(uint8_t)suffix2 {
    uint8_t* array = (uint8_t*)data.bytes;
    int length = (int)data.length;
    NSInteger prefixPos = NSNotFound;
    NSInteger suffix1Pos = NSNotFound;
    NSInteger suffix2Pos = NSNotFound;
    BOOL is_DataCarry = NO;
    int tagLen = 0;
    int POS_DATA = 3;
    int POS_RESULT_DATA = 10;
    int POS_TAG = 0;
    BOOL barcodeflag = NO;
    for (int i = 0; i < length - 1; i++) {
        if (prefixPos == NSNotFound) {
            if (array[i] == prefix) {
                prefixPos = i;
                if(array[i+1] == 0x65)
                {
                    if(array[i+2] == 0x43){
                        barcodeflag = YES;
                    } else {
                        barcodeflag = NO;
                    }
                    switch (array[i+2]) {
                        case 0x41:
                        {
                            is_DataCarry = YES;
                            POS_TAG = i+POS_RESULT_DATA;
                            uint8_t len = array[POS_TAG];
                            tagLen = (len & 0x7E) + 1;
                            break;
                        }
                        case 0x46:
                        case 0x54:
                        {
                            is_DataCarry = YES;
                            POS_TAG = i+POS_DATA;
                            uint8_t len = array[POS_TAG];
                            tagLen = (len & 0x7E) + 1;
                            break;
                        }
                    }
                }

                continue;
            }
        } else {
            if (barcodeflag == YES){
                if (array[i] == suffix1 && array[i+1] == suffix2 && array[i+2] == 0x7e && array[i+3] == 0x41 && array[i+4] == 0x73) {
                    if(is_DataCarry == YES)
                    {
                        if( i < (POS_TAG+tagLen))
                        {
                            continue;
                        }
                    }
                    suffix1Pos = i;
                    suffix2Pos = i+1;
                    break;
                }
            }else {
                if (array[i] == suffix1 && array[i+1] == suffix2) {
                    suffix1Pos = i;
                    suffix2Pos = i+1;
                    if(is_DataCarry == YES)
                    {
                        if( i < (POS_TAG+tagLen))
                        {
                            break;
                        }
                    }
                    break;
                }
            }
        }
    }
    NSRange range;
    if (prefixPos == NSNotFound && suffix2Pos == NSNotFound)
        range = NSMakeRange(prefixPos, suffix2Pos);
    else if (prefixPos != NSNotFound && suffix2Pos == NSNotFound)
        range = NSMakeRange(prefixPos, NSNotFound);
    else if (prefixPos == NSNotFound && suffix2Pos != NSNotFound)
        range = NSMakeRange(prefixPos, suffix2Pos);
    else
        range = NSMakeRange(prefixPos, (suffix2Pos - prefixPos) + 1);
    return range;
}
- (NSInteger)getErrorCode:(NSData *)data findPosition:(int)position {
    int length = 4;
    if ( (int)[data length] - position < 4) {
        length = (int)[data length] - position;
    }
    NSRange range = NSMakeRange(position, length);
    NSData *value = [data subdataWithRange:range];
    NSString *convValue = [[NSString alloc]initWithData:value encoding:NSASCIIStringEncoding];
    unsigned res = 0;
    [[NSScanner scannerWithString:convValue]scanHexInt:&res];
    return res;
}
@end
