//
//  LongRange2DManager.m
//  LongRange2DDemo
//
//  Created by mac on 2018/12/24.
//  Copyright © 2018年 zyl. All rights reserved.
//

#import "LongRange2DManager.h"
#import "SystemSetting.h"
#define DeviceModel @"com.asreader.r202g"

@implementation LongRange2DManager{
    NSTimer           *batteryTimer;
    SystemSetting     *systemSetting;
    LongRange2D       *_mLongRange2D;
    LongRange2DDevice *_longRange2DDevice;
}

static LongRange2DManager * sharedLongRange2DManager;
+ (LongRange2DManager *)sharedLongRange2DManager{
    if (!sharedLongRange2DManager) {
        sharedLongRange2DManager = [[LongRange2DManager alloc] init];
    }
    return sharedLongRange2DManager;
}
- (instancetype)init
{
    self = [super init];
    if (self) {
        NSNotificationCenter *center = [NSNotificationCenter defaultCenter];
        [center addObserver:self selector:@selector(connected) name:LongRange2DDeviceDidConnectedNotification object:nil];
        [center addObserver:self selector:@selector(disconnected) name:LongRange2DDeviceDidDisonnectedNotification object:nil];
        [AsRingDeviceSDK sharedAsRingDeviceSDK];
        systemSetting = [SystemSetting sharedSystemSetting];
        _longRange2DDevice = [[LongRange2DDevice alloc] initWithDeviceModel:DeviceModel];
        _isConnected = NO;
    }
    return self;
}
- (BOOL)centerDecoding{
    return _mLongRange2D.centerDecoding;
}
- (void)setCenterDecoding:(BOOL)centerDecoding{
    [_mLongRange2D setCenterDecoding:centerDecoding];
}
- (void)disconnected{
    _isConnected = NO;
    _mLongRange2D = nil;
    if (_delegate && [_delegate respondsToSelector:@selector(LongRange2DConnected:)]) {
        [_delegate LongRange2DConnected:_isConnected];
    }
}
- (void)connected{
    _mLongRange2D = [[LongRange2D alloc] initWithLongRange2DDevice:_longRange2DDevice delegate:self];
}
- (void)longRange2DInitialized:(LongRange2D *)longRange2D{
    _isConnected = YES;
    NSLog(@"连接成功");
    if (_delegate && [_delegate respondsToSelector:@selector(LongRange2DConnected:)]) {
        [_delegate LongRange2DConnected:_isConnected];
    }
}
- (void)updateDeviceState:(LongRange2DResultCode)error{
    if (_delegate && [_delegate respondsToSelector:@selector(updateDeviceState:)]) {
        [_delegate updateDeviceState:error];
    }
}
- (void)changedActionState:(LongRange2DCommandType)action resultCode:(NSInteger)resultCode{
    if (_delegate && [_delegate respondsToSelector:@selector(changedActionState:resultCode:)]) {
        [_delegate changedActionState:action resultCode:resultCode];
    }
}
- (void)receivedBarcodeString:(NSString *)barcodeString
                  barcodeType:(LongRange2DBarcodeTypeOptions)barcodeType{
    if (_delegate && [_delegate respondsToSelector:@selector(receivedBarcodeString:barcodeType:)]) {
           [_delegate receivedBarcodeString:barcodeString barcodeType:barcodeType];
       }
}
- (void)receivedBarcodeData:(NSData *)barcodeData
                barcodeType:(LongRange2DBarcodeTypeOptions)barcodeType{
    if (_delegate && [_delegate respondsToSelector:@selector(receivedBarcodeData:barcodeType:)]) {
        [_delegate receivedBarcodeData:barcodeData barcodeType:barcodeType];
    }
}
- (BOOL)onAsReaderTriggerKeyEvent:(BOOL)status{
    if (_delegate && [_delegate respondsToSelector:@selector(onAsReaderTriggerKeyEvent:)]) {
       return [_delegate onAsReaderTriggerKeyEvent:status];
    }
    return NO;
}

- (void)receivedData:(NSData*)data{
    if (_delegate && [_delegate respondsToSelector:@selector(receivedData:)]) {
       return [_delegate receivedData:data];
    }
}

- (void)startMonitoringBattery:(BOOL)isStart{
    [self updateBattery];
    float periodicTime = [[[SystemSetting sharedSystemSetting] getBatteryInterval] intValue];


    if(batteryTimer){
        [batteryTimer invalidate];
        batteryTimer = nil;
    }
    
    if(isStart){
        if (periodicTime == 0) {
            if (_delegate && [_delegate respondsToSelector:@selector(batteryStatus:)]) {
                [_delegate batteryStatus:[_mLongRange2D getBatteryStatus]];
            }
            return;
        }
        batteryTimer = [NSTimer scheduledTimerWithTimeInterval:periodicTime target:self selector:@selector(updateBattery) userInfo:nil repeats:YES];
        [[NSRunLoop mainRunLoop] addTimer:batteryTimer forMode:NSRunLoopCommonModes];
        [batteryTimer fire];
    }
}

- (void)updateBattery{
    if (_delegate && [_delegate respondsToSelector:@selector(batteryStatus:)]) {
        [_delegate batteryStatus:[_mLongRange2D getBatteryStatus]];
    }
}

+ (MBProgressHUD *)showGlobalProgressHUDWithTitle:(NSString *)title {
    UIViewController *result = nil;
    UIWindow * window = [[UIApplication sharedApplication] keyWindow];
    if (window.windowLevel != UIWindowLevelNormal)
    {
        NSArray *windows = [[UIApplication sharedApplication] windows];
        for(UIWindow * tmpWin in windows)
        {
            if (tmpWin.windowLevel == UIWindowLevelNormal)
            {
                window = tmpWin;
                break;
            }
        }
    }
    UIView *frontView = [[window subviews] objectAtIndex:0];
    id nextResponder = [frontView nextResponder];
    
    if ([nextResponder isKindOfClass:[UIViewController class]])
        result = nextResponder;
    else
        result = window.rootViewController;
    MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:[result view] animated:YES];
    hud.label.text = title;
    return hud;
}

+ (MBProgressHUD *)showProgressBarHUDWithTitle:(NSString *)title{
    UIViewController *result = nil;
    UIWindow * window = [[UIApplication sharedApplication] keyWindow];
    if (window.windowLevel != UIWindowLevelNormal)
    {
        NSArray *windows = [[UIApplication sharedApplication] windows];
        for(UIWindow * tmpWin in windows)
        {
            if (tmpWin.windowLevel == UIWindowLevelNormal)
            {
                window = tmpWin;
                break;
            }
        }
    }
    UIView *frontView = [[window subviews] objectAtIndex:0];
    id nextResponder = [frontView nextResponder];

    if ([nextResponder isKindOfClass:[UIViewController class]])
        result = nextResponder;
    else
        result = window.rootViewController;
    MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:[result view] animated:YES];
    hud.mode = MBProgressHUDModeDeterminate;
    hud.label.text = title;
    return hud;
}

+ (void)dismissGlobalHUD {
    UIViewController *result = nil;
    UIWindow * window = [[UIApplication sharedApplication] keyWindow];
    if (window.windowLevel != UIWindowLevelNormal)
    {
        NSArray *windows = [[UIApplication sharedApplication] windows];
        for(UIWindow * tmpWin in windows)
        {
            if (tmpWin.windowLevel == UIWindowLevelNormal)
            {
                window = tmpWin;
                break;
            }
        }
    }
    UIView *frontView = [[window subviews] objectAtIndex:0];
    id nextResponder = [frontView nextResponder];
    
    if ([nextResponder isKindOfClass:[UIViewController class]])
        result = nextResponder;
    else
        result = window.rootViewController;
    [MBProgressHUD hideHUDForView:[result view]  animated:YES];
}
- (NSString *)getSDKVersion{
    return  [_longRange2DDevice getSDKVersion];
}
- (NSString *)getDeviceModel{
    return  [_longRange2DDevice deviceModel];
}
- (NSString *)getAddress{
    return  [_longRange2DDevice getAddress];
}
- (LongRange2DCommandType)getAction{
    return [_mLongRange2D getAction];
}
- (NSString *)getSerialNumber{
    return [_mLongRange2D getSerialNumber];
}
- (NSString *)getFirmwareVersion{
    return [_mLongRange2D getFirmwareVersion];
}
- (int)getBatteryStatus{
    return [_mLongRange2D getBatteryStatus];
}

- (LongRange2DResultCode)startScan{
    return [_mLongRange2D startScan];
}
- (LongRange2DResultCode)stopScan{
    return [_mLongRange2D stopScan];
}
- (LongRange2DResultCode)setBarcodeParam:(LongRange2DParamValue *)paramData{
    return [_mLongRange2D setBarcodeParam:paramData];
}
- (LongRange2DParamValue *)getBarcodeParam:(LongRange2DParamNameOptions)paramName{
    return [_mLongRange2D getBarcodeParam:paramName];
}
- (LongRange2DResultCode)startBeepWithBeepTime:(int)beepTime{
    return [_mLongRange2D startBeepWithBeepTime:beepTime];
}
- (LongRange2DResultCode)startVibratorWithVibratorTime:(int)vibratorTime{
    return [_mLongRange2D startVibratorWithVibratorTime:vibratorTime];
}
- (LongRange2DResultCode)setDefaultScannerPrama{
    return [_mLongRange2D setDefaultScannerPrama];
}
- (NSString *)getMessageWithResultCode:(LongRange2DResultCode)resultCode{
    if (!_mLongRange2D) {
        return @"Other Error";
    }
    return [_mLongRange2D getMessageWithResultCode:resultCode];
}
- (BOOL)sendData:(NSData*)data{
    if (!_mLongRange2D) {
        return NO;
    }
  
    return [_mLongRange2D sendData:data];
}
- (void)setBeep:(LongRange2DBeepState)beepState{
    _mLongRange2D.beep = beepState;
}
- (void)setVibrator:(LongRange2DVibratorState)vibratorState{
    _mLongRange2D.vibrator = vibratorState;
}
- (void)setSleepTime:(int)sleepTime{
    _mLongRange2D.sleepTime = sleepTime;
}
- (void)setAutoOffTime:(int)autoOffTime{
    _mLongRange2D.autoOffTime = autoOffTime;
}
- (void)setScannerTimeOut:(int)scannerTimeOut{
    _mLongRange2D.scannerTimeOut = scannerTimeOut;
}
- (LongRange2DBeepState)getBeep{
    return _mLongRange2D.beep;
}
- (LongRange2DVibratorState)getVibrator{
    return _mLongRange2D.vibrator;
}
- (int)getSleepTime{
    return _mLongRange2D.sleepTime;
}
- (int)getAutoOffTime{
    return _mLongRange2D.autoOffTime;
}
- (int)getScannerTimeOut{
    return _mLongRange2D.scannerTimeOut;
}
- (void)startUpdateFirmware {
//    [_longRange2DDevice startUpdateFirmware];
}
- (void)stopUpdateFirmware {
//    [_longRange2DDevice stopUpdateFirmware];
}
@end
