#import "MainViewController.h"
#import "SystemSetting.h"
#define BATTERY_NONE            0
#define BATTERY_QUARTER         1
#define BATTERY_HALF            2
#define BATTERY_THIRD_QUARTER   3
#define BATTERY_FULL            4

#define CONNECTION_OFF          0
#define CONNECTION_ON           1

@interface MainViewController ()

@end

@implementation MainViewController{
    LongRange2DManager *_longRange2DManager;
    UIAlertController *_currentAlertController;
    UIView * backView;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    _longRange2DManager = [LongRange2DManager sharedLongRange2DManager];
    NSDictionary *infoDictionary = [[NSBundle mainBundle] infoDictionary];
    NSString *majorVersion = [infoDictionary objectForKey:@"CFBundleShortVersionString"];
    NSString *minorVersion = [infoDictionary objectForKey:@"CFBundleVersion"];
    self.labelDemoVer.text = [NSString stringWithFormat:@"%@ Build %@", majorVersion, minorVersion];
    self.snLabel.text = @"";
    
    self.buzzerTextField.text =[NSString stringWithFormat:@"%d",[[SystemSetting sharedSystemSetting] getBeepTime]];
    self.vibratorTextField.text = [NSString stringWithFormat:@"%d",[[SystemSetting sharedSystemSetting] getVibratorTime]];
    self.buzzerTextField.delegate = self;
    self.vibratorTextField.delegate = self;
    backView = [[UIView alloc] init];
    backView.backgroundColor = [UIColor grayColor];
    backView.alpha = 0.5;
    backView.frame = CGRectMake(0, 0, self.view.frame.size.width, self.view.frame.size.height);
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    _longRange2DManager.delegate = self;
    UITapGestureRecognizer *tapGestureRecognizer = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(keyboardHide:)];
    tapGestureRecognizer.cancelsTouchesInView = NO;
    [self.view addGestureRecognizer:tapGestureRecognizer];
}
- (void) keyboardHide : (NSNotification*)notification {
    [backView removeFromSuperview];
    [self.vibratorTextField resignFirstResponder];
    [self.buzzerTextField resignFirstResponder];
    if ([self.buzzerTextField.text intValue] == 0) {
        self.buzzerTextField.text = @"1";
    }
    if ( [self.buzzerTextField.text intValue]  >= 5000) {
        self.buzzerTextField.text = @"4999";

        UIAlertController * alertController = [UIAlertController alertControllerWithTitle:@"Beep Value" message:@"Please enter the value (1~4999)"  preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction * actionCancel = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleCancel handler:nil];
        [alertController addAction:actionCancel];
        [self.navigationController presentViewController:alertController animated:NO completion:nil];
    }
    if ([self.vibratorTextField.text intValue] == 0) {
        self.vibratorTextField.text = @"1";
    }
    if ( [self.vibratorTextField.text intValue]  >= 5000) {
        self.vibratorTextField.text = @"4999";

        UIAlertController * alertController = [UIAlertController alertControllerWithTitle:@"Vibrator Value" message:@"Please enter the value (1~4999)"  preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction * actionCancel = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleCancel handler:nil];
        [alertController addAction:actionCancel];
        [self.navigationController presentViewController:alertController animated:NO completion:nil];
    }
    [[SystemSetting sharedSystemSetting] setVibratorTime:[self.vibratorTextField.text intValue]];
    [[SystemSetting sharedSystemSetting] setBeepTime:[self.buzzerTextField.text intValue]];
}
- (void)viewDidAppear:(BOOL)animated{
    if ([self.navigationController respondsToSelector:@selector(interactivePopGestureRecognizer)])
        self.navigationController.interactivePopGestureRecognizer.enabled = NO;
    [self reloadUI];
}
- (void)reloadUI{
    if  (_longRange2DManager.isConnected){
         NSLog(@"Loading shows");
        [LongRange2DManager showGlobalProgressHUDWithTitle:@"Loading..."];
        self.asRingPlusVersion.text = [[AsRingDeviceSDK sharedAsRingDeviceSDK] getAsRingFirmwareRevision];
        [_longRange2DManager setDelegate:self];
        self.labelFirmwareVer.text = [_longRange2DManager getFirmwareVersion];
        [_longRange2DManager startMonitoringBattery:YES];
        self.snLabel.text = [_longRange2DManager getSerialNumber];
        [self enableAllButtons];
        NSLog(@"Loading hide");
        [LongRange2DManager dismissGlobalHUD];
    }else{
        [self disableAllUIs];
    }
}
- (void)viewDidDisappear:(BOOL)animated{
    [super viewDidDisappear:animated];
    [_longRange2DManager startMonitoringBattery:NO];
}
- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

#pragma mark - Reader Event Handler Methods
- (void)LongRange2DConnected:(BOOL)isConnected {
    NSLog(@"%s", __PRETTY_FUNCTION__);
    dispatch_async(dispatch_get_main_queue(), ^{
        if (isConnected) {
            NSLog(@"Test connect = YES");
            [self reloadUI];
            if ([[SystemSetting sharedSystemSetting] needUpdateFirmwareVersion:self.labelFirmwareVer.text]) {
                UIAlertController * alertController = [UIAlertController alertControllerWithTitle:@"" message:@"Update the firmware to the latest to use all functions." preferredStyle:(UIAlertControllerStyleAlert)];
                UIAlertAction * cancelAction = [UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:^(UIAlertAction * _Nonnull action) {
                    self->_currentAlertController = nil;
                }];
                UIAlertAction * updateAction = [UIAlertAction actionWithTitle:@"Update" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
                    self->_currentAlertController = nil;
                    [self performSegueWithIdentifier:@"GotoUpdateFirmware" sender:self];
                }];
                [alertController addAction:cancelAction];
                [alertController addAction:updateAction];
                if (self->_currentAlertController) {
                    [self->_currentAlertController dismissViewControllerAnimated:YES completion:nil];
                }
                [self presentViewController:alertController animated:YES completion:nil];
                self->_currentAlertController = alertController;
            }
        }else{
            NSLog(@"Test connect = NO");
            if (self->_currentAlertController) {
                [self->_currentAlertController dismissViewControllerAnimated:YES completion:nil];
            }
            [self disableAllUIs];
            [LongRange2DManager dismissGlobalHUD];
        }
    });
}

- (void)updateDeviceState:(LongRange2DResultCode)error {
    if (error != LongRange2DResultNoError) {
        NSString * message = @"";
        NSString * title  = @"";
        title   = @"AsRingDevice";
        message = @"AsRing device is not available on this device";   
        NSLog(@"MV - updateDeviceState %@",message);

    }
}
- (void)changedActionState:(LongRange2DCommandType)action resultCode:(NSInteger)resultCode {
    NSLog(@"MV - changedActionState");
}

- (void)batteryStatus:(int)status{
    [self changeBatteryImage:status];
}

#pragma mark - Custom
- (void)enableAllButtons {
    for (MainButton *btn in self.buttonCollection) {
        [btn enableButton];
    }
    [self changeLogoImage:CONNECTION_ON];
    
    [self.buzzerButton setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
    [self.vibratorButton setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
    self.buzzerTextField.text =[NSString stringWithFormat:@"%d",[[SystemSetting sharedSystemSetting] getBeepTime]];
    self.vibratorTextField.text = [NSString stringWithFormat:@"%d",[[SystemSetting sharedSystemSetting] getVibratorTime]];
    
    [self.buzzerTextField setEnabled:YES];
    [self.vibratorTextField setEnabled:YES];
    [self.buzzerButton setEnabled:YES];
    [self.vibratorButton setEnabled:YES];
}

- (void)disableAllUIs {
    [_longRange2DManager startMonitoringBattery:NO];
    [self keyboardHide:nil];
    self.asRingPlusVersion.text = @"";
    self.labelFirmwareVer.text = @"";
    self.snLabel.text = @"";

    [self.buzzerButton setTitleColor:[UIColor grayColor] forState:UIControlStateNormal];
    [self.vibratorButton setTitleColor:[UIColor grayColor] forState:UIControlStateNormal];
    self.buzzerTextField.text = [NSString stringWithFormat:@"%d",[[SystemSetting sharedSystemSetting] getBeepTime]];
    self.vibratorTextField.text = [NSString stringWithFormat:@"%d",[[SystemSetting sharedSystemSetting] getVibratorTime]];
    [self.buzzerTextField setEnabled:NO];
    [self.vibratorTextField setEnabled:NO];
    [self.buzzerButton setEnabled:NO];
    [self.vibratorButton setEnabled:NO];

    for (MainButton *btn in self.buttonCollection) {
        [btn disableButton];
    }
    [self changeBatteryImage:BATTERY_NONE];
    [self changeLogoImage:CONNECTION_OFF];
}

- (void)changeBatteryImage:(NSUInteger)batteryStatus {
    if (batteryStatus == BATTERY_NONE) {
        [self.batteryImage setImage:[UIImage imageNamed:@"battery0"]];
    } else if (batteryStatus == BATTERY_QUARTER) {
        [self.batteryImage setImage:[UIImage imageNamed:@"battery1"]];
    } else if (batteryStatus == BATTERY_HALF) {
        [self.batteryImage setImage:[UIImage imageNamed:@"battery2"]];
    } else if (batteryStatus == BATTERY_THIRD_QUARTER) {
        [self.batteryImage setImage:[UIImage imageNamed:@"battery3"]];
    } else if (batteryStatus == BATTERY_FULL) {
        [self.batteryImage setImage:[UIImage imageNamed:@"battery4"]];
    }
}

- (void)changeLogoImage:(NSUInteger)connectionStatus {
    if (connectionStatus == CONNECTION_OFF) {
        [self.logoImage setImage:[UIImage imageNamed:@"asreader_logo_off"]];
    } else if (connectionStatus == CONNECTION_ON) {
		[self.logoImage setImage:[UIImage imageNamed:@"asreader_logo_on"]];
    }
}

- (IBAction)onClickVibratorButton:(id)sender {
    if ([self.vibratorTextField.text intValue] > 0 && [self.vibratorTextField.text intValue] < 5000 ) {
        [[SystemSetting sharedSystemSetting] setVibratorTime:[self.vibratorTextField.text intValue]];
        LongRange2DResultCode  rCode = [_longRange2DManager startVibratorWithVibratorTime:[self.vibratorTextField.text intValue]];
        if (rCode == LongRange2DResultOperationFailed) {
            NSLog(@"Vibrator Start ERROR!");
        }
    }else{
        UIAlertController * alertController = [UIAlertController alertControllerWithTitle:@"Vibrator Value" message:@"Please enter the value (1~4999)" preferredStyle:(UIAlertControllerStyleAlert)];
        UIAlertAction * action = [UIAlertAction actionWithTitle:@"OK" style:(UIAlertActionStyleDefault) handler:^(UIAlertAction * _Nonnull action) {
        }];
        [alertController addAction:action];
        [self presentViewController:alertController animated:YES completion:nil];
    }
}
- (IBAction)onClickBuzzerButton:(id)sender {
    if ([self.buzzerTextField.text intValue] > 0 && [self.buzzerTextField.text intValue] < 5000 ) {
        [[SystemSetting sharedSystemSetting] setBeepTime:[self.buzzerTextField.text intValue]];
       LongRange2DResultCode  rCode = [_longRange2DManager  startBeepWithBeepTime:[self.buzzerTextField.text intValue]];
        if (rCode == LongRange2DResultOperationFailed) {
            NSLog(@"Beep Start ERROR!");
        }
    }else{
        UIAlertController * alertController = [UIAlertController alertControllerWithTitle:@"Beep Value" message:@"Please enter the value (1~4999)" preferredStyle:(UIAlertControllerStyleAlert)];
        UIAlertAction * action = [UIAlertAction actionWithTitle:@"OK" style:(UIAlertActionStyleDefault) handler:^(UIAlertAction * _Nonnull action) {
        }];
        [alertController addAction:action];
        [self presentViewController:alertController animated:YES completion:nil];
    }
}
- (void)textFieldDidBeginEditing:(UITextField *)textField{
    [self.view addSubview:backView];
    if ([self.buzzerTextField isFirstResponder]) {
        self.buzzerTextField.keyboardType = UIKeyboardTypeNumberPad;
    }else if ([self.vibratorTextField isFirstResponder]) {
        self.vibratorTextField.keyboardType = UIKeyboardTypeNumberPad;
    }
}
- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string{
    if ([[textField.text stringByAppendingString:string]intValue] <= 0) {
        textField.text = @"1";
    }
    return YES;
}
@end
